@@ -63,17 +63,17 @@ easily and automatically extracted with the Sphinx extensions in the
 Basic Properties
 ----------------
 
-%s
+{basic_properties}
 
 Container Properties
 --------------------
 
-%s
+{container_properties}
 
 DataSpec Properties
 -------------------
 
-%s
+{dataspec_properties}
 
 Helpers
 ~~~~~~~
@@ -106,25 +106,18 @@ from six import string_types, iteritems
 
 from ..colors import RGB
 from ..util.dependencies import import_optional
-from ..util.deprecation import deprecated
-from ..util.serialization import transform_column_source_data, decode_base64_dict
-from ..util.string import nice_join
+from ..util.serialization import convert_datetime_type, decode_base64_dict, transform_column_source_data
+from ..util.string import nice_join, format_docstring
+
 from .property.bases import ContainerProperty, DeserializationError, ParameterizedProperty, Property, PrimitiveProperty
+from .property.containers import PropertyValueColumnData, PropertyValueDict, PropertyValueList
 from .property.descriptor_factory import PropertyDescriptorFactory
-from .property.descriptors import BasicPropertyDescriptor, DataSpecPropertyDescriptor, UnitsSpecPropertyDescriptor
+from .property.descriptors import (BasicPropertyDescriptor, ColumnDataPropertyDescriptor, DataSpecPropertyDescriptor,
+                                   UnitsSpecPropertyDescriptor)
 from . import enums
 
 pd = import_optional('pandas')
 
-# TODO: this should really be moved elsewhere, has_props.py or model.py
-def abstract(cls):
-    from .has_props import HasProps
-    ''' A phony decorator to mark abstract base classes. '''
-    if not issubclass(cls, HasProps):
-        raise TypeError("%s is not a subclass of HasProps" % cls.__name__)
-
-    return cls
-
 bokeh_bool_types = (bool,)
 try:
     import numpy as np
@@ -476,9 +469,9 @@ class Instance(Property):
                 raise ValueError("expected an instance of type %s, got %s of type %s" %
                     (self.instance_type.__name__, value, type(value).__name__))
 
-    def _has_stable_default(self):
+    def _may_have_unstable_default(self):
         # because the instance value is mutable
-        return False
+        return True
 
     def _sphinx_type(self):
         fullname = "%s.%s" % (self.instance_type.__module__, self.instance_type.__name__)
@@ -645,6 +638,9 @@ class Either(ParameterizedProperty):
             return self._type_params[0]._raw_default()
         default = kwargs.get("default", choose_default)
         super(Either, self).__init__(default=default, help=help)
+        self.alternatives = []
+        for tp in self._type_params:
+            self.alternatives.extend(tp.alternatives)
 
     # TODO (bev) get rid of this?
     def __or__(self, other):
@@ -681,6 +677,10 @@ class Either(ParameterizedProperty):
         if not (value is None or any(param.is_valid(value) for param in self.type_params)):
             raise ValueError("expected an element of either %s, got %r" % (nice_join(self.type_params), value))
 
+    # TODO (bev) implement this
+    # def _may_have_unstable_default(self):
+    #     return any(tp._may_have_unstable_default() for tp in self.type_params)
+
     def _sphinx_type(self):
         return self._sphinx_prop_link() + "( %s )" % ", ".join(x._sphinx_type() for x in self.type_params)
 
@@ -1034,10 +1034,10 @@ class Angle(Float):
     pass
 
 class Date(Property):
-    ''' Accept Date (not datetime) values.
+    ''' Accept Date (but not DateTime) values.
 
     '''
-    def __init__(self, default=datetime.date.today(), help=None):
+    def __init__(self, default=None, help=None):
         super(Date, self).__init__(default=default, help=help)
 
     def transform(self, value):
@@ -1135,35 +1135,6 @@ class TimeDelta(Property):
 
         raise ValueError("Expected a timedelta instance, got %r" % value)
 
-class TitleProp(Either):
-    ''' Accept a title value for a plot (possibly transforming a plain string).
-
-    .. note::
-        This property exists only to support a deprecation, and will be removed
-        in the future once the deprecation is completed.
-
-    '''
-    def __init__(self, default=None, help=None):
-        types = (Instance('bokeh.models.annotations.Title'), String)
-        super(TitleProp, self).__init__(*types, default=default, help=help)
-
-    def transform(self, value):
-        if isinstance(value, str):
-            from bokeh.models.annotations import Title
-            deprecated('''Setting Plot property 'title' using a string was deprecated in 0.12.0,
-            and will be removed. The title is now an object on Plot (which holds all of it's
-            styling properties). Please use Plot.title.text instead.
-
-            SERVER USERS: If you were using plot.title to have the server update the plot title
-            in a callback, you MUST update to plot.title.text as the title object cannot currently
-            be replaced after initialization.
-            ''')
-            value = Title(text=value)
-        return value
-
-    def _sphinx_type(self):
-        return self._sphinx_prop_link()
-
 #------------------------------------------------------------------------------
 # Container properties
 #------------------------------------------------------------------------------
@@ -1234,6 +1205,19 @@ class List(Seq):
         # optional values. Also in Dict.
         super(List, self).__init__(item_type, default=default, help=help)
 
+    @classmethod
+    def wrap(cls, value):
+        ''' Some property types need to wrap their values in special containers, etc.
+
+        '''
+        if isinstance(value, list):
+            if isinstance(value, PropertyValueList):
+                return value
+            else:
+                return PropertyValueList(value)
+        else:
+            return value
+
     @classmethod
     def _is_seq(cls, value):
         return isinstance(value, list)
@@ -1289,10 +1273,22 @@ class Dict(ContainerProperty):
                     all(self.keys_type.is_valid(key) and self.values_type.is_valid(val) for key, val in iteritems(value))):
                 raise ValueError("expected an element of %s, got %r" % (self, value))
 
+    @classmethod
+    def wrap(cls, value):
+        ''' Some property types need to wrap their values in special containers, etc.
+
+        '''
+        if isinstance(value, dict):
+            if isinstance(value, PropertyValueDict):
+                return value
+            else:
+                return PropertyValueDict(value)
+        else:
+            return value
+
     def _sphinx_type(self):
         return self._sphinx_prop_link() + "( %s, %s )" % (self.keys_type._sphinx_type(), self.values_type._sphinx_type())
 
-
 class ColumnData(Dict):
     ''' Accept a Python dictionary suitable as the ``data`` attribute of a
     :class:`~bokeh.models.sources.ColumnDataSource`.
@@ -1302,6 +1298,23 @@ class ColumnData(Dict):
 
     '''
 
+    def make_descriptors(self, base_name):
+        ''' Return a list of ``ColumnDataPropertyDescriptor`` instances to
+        install on a class, in order to delegate attribute access to this
+        property.
+
+        Args:
+            base_name (str) : the name of the property these descriptors are for
+
+        Returns:
+            list[ColumnDataPropertyDescriptor]
+
+        The descriptors returned are collected by the ``MetaHasProps``
+        metaclass and added to ``HasProps`` subclasses during class creation.
+        '''
+        return [ ColumnDataPropertyDescriptor(base_name, self) ]
+
+
     def from_json(self, json, models=None):
         ''' Decodes column source data encoded as lists or base64 strings.
         '''
@@ -1327,10 +1340,22 @@ class ColumnData(Dict):
                 new_data[key] = self.values_type.from_json(value, models)
         return new_data
 
-
     def serialize_value(self, value):
         return transform_column_source_data(value)
 
+    @classmethod
+    def wrap(cls, value):
+        ''' Some property types need to wrap their values in special containers, etc.
+
+        '''
+        if isinstance(value, dict):
+            if isinstance(value, PropertyValueColumnData):
+                return value
+            else:
+                return PropertyValueColumnData(value)
+        else:
+            return value
+
 class Tuple(ContainerProperty):
     ''' Accept Python tuple values.
 
@@ -1466,21 +1491,21 @@ class DataSpec(Either):
             color = ColorSpec(help="docs for color") # defaults to None
 
     '''
-    def __init__(self, typ, default, help=None):
+    def __init__(self, key_type, value_type, default, help=None):
         super(DataSpec, self).__init__(
             String,
             Dict(
-                String,
+                key_type,
                 Either(
                     String,
                     Instance('bokeh.models.transforms.Transform'),
-                    Instance('bokeh.models.mappers.ColorMapper'),
-                    typ)),
-            typ,
+                    Instance('bokeh.models.expressions.Expression'),
+                    value_type)),
+            value_type,
             default=default,
             help=help
         )
-        self._type = self._validate_type_param(typ)
+        self._type = self._validate_type_param(value_type)
 
     # TODO (bev) add stricter validation on keys
 
@@ -1490,7 +1515,7 @@ class DataSpec(Either):
         property.
 
         Args:
-            name (str) : the name of the property these descriptors are for
+            base_name (str) : the name of the property these descriptors are for
 
         Returns:
             list[DataSpecPropertyDescriptor]
@@ -1517,14 +1542,22 @@ class DataSpec(Either):
         if isinstance(val, string_types):
             return dict(field=val)
 
-        # Must be dict, return as-is
-        return val
+        # Must be dict, return a new dict
+        return dict(val)
 
     def _sphinx_type(self):
         return self._sphinx_prop_link()
 
+_ExprFieldValueTransform = Enum("expr", "field", "value", "transform")
+
 class NumberSpec(DataSpec):
-    ''' A |DataSpec| property that accepts numeric fixed values.
+    ''' A |DataSpec| property that accepts numeric and datetime fixed values.
+
+    By default, date and datetime values are immediately converted to
+    milliseconds since epoch. It it possible to disable processing of datetime
+    values by passing ``accept_datetime=False``.
+
+    Timedelta values are interpreted as absolute milliseconds.
 
     .. code-block:: python
 
@@ -1533,8 +1566,12 @@ class NumberSpec(DataSpec):
         m.location = "foo" # field
 
     '''
-    def __init__(self, default=None, help=None):
-        super(NumberSpec, self).__init__(Float, default=default, help=help)
+    def __init__(self, default=None, help=None, key_type=_ExprFieldValueTransform, accept_datetime=True):
+        super(NumberSpec, self).__init__(key_type, Float, default=default, help=help)
+        self.accepts(TimeDelta, convert_datetime_type)
+        if accept_datetime:
+            self.accepts(Datetime, convert_datetime_type)
+
 
 class StringSpec(DataSpec):
     ''' A |DataSpec| property that accepts string fixed values.
@@ -1551,8 +1588,8 @@ class StringSpec(DataSpec):
         m.title = "foo"        # field
 
     '''
-    def __init__(self, default, help=None):
-        super(StringSpec, self).__init__(List(String), default=default, help=help)
+    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform):
+        super(StringSpec, self).__init__(key_type, List(String), default=default, help=help)
 
     def prepare_value(self, cls, name, value):
         if isinstance(value, list):
@@ -1581,23 +1618,37 @@ class FontSizeSpec(DataSpec):
     https://drafts.csswg.org/css-values/#lengths
 
     '''
-    _font_size_re = re.compile("^[0-9]+(\.[0-9]+)?(%|em|ex|ch|ic|rem|vw|vh|vi|vb|vmin|vmax|cm|mm|q|in|pc|pt|px)$", re.I)
+    _font_size_re = re.compile(r"^[0-9]+(.[0-9]+)?(%|em|ex|ch|ic|rem|vw|vh|vi|vb|vmin|vmax|cm|mm|q|in|pc|pt|px)$", re.I)
 
-    def __init__(self, default, help=None):
-        super(FontSizeSpec, self).__init__(List(String), default=default, help=help)
+    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform):
+        super(FontSizeSpec, self).__init__(key_type, List(String), default=default, help=help)
 
     def prepare_value(self, cls, name, value):
         if isinstance(value, string_types) and self._font_size_re.match(value) is not None:
             value = dict(value=value)
         return super(FontSizeSpec, self).prepare_value(cls, name, value)
 
+    def validate(self, value):
+        super(FontSizeSpec, self).validate(value)
+
+        if isinstance(value, dict) and 'value' in value:
+            value = value['value']
+
+        if isinstance(value, string_types):
+            if len(value) == 0:
+                raise ValueError("empty string is not a valid font size value")
+            elif value[0].isdigit() and self._font_size_re.match(value) is None:
+                raise ValueError("%r is not a valid font size value" % value)
+
+_ExprFieldValueTransformUnits = Enum("expr", "field", "value", "transform", "units")
+
 class UnitsSpec(NumberSpec):
     ''' A |DataSpec| property that accepts numeric fixed values, and also
     provides an associated units property to store units information.
 
     '''
     def __init__(self, default, units_type, units_default, help=None):
-        super(UnitsSpec, self).__init__(default=default, help=help)
+        super(UnitsSpec, self).__init__(default=default, help=help, key_type=_ExprFieldValueTransformUnits)
         self._units_type = self._validate_type_param(units_type)
         # this is a hack because we already constructed units_type
         self._units_type.validate(units_default)
@@ -1708,7 +1759,7 @@ class DataDistanceSpec(NumberSpec):
         return super(DataDistanceSpec, self).prepare_value(cls, name, value)
 
     def to_serializable(self, obj, name, val):
-        d = super(ScreenDistanceSpec, self).to_serializable(obj, name, val)
+        d = super(DataDistanceSpec, self).to_serializable(obj, name, val)
         d["units"] = "data"
         return d
 
@@ -1737,8 +1788,8 @@ class ColorSpec(DataSpec):
         m.color = field("firebrick")       # field (named "firebrick")
 
     '''
-    def __init__(self, default, help=None):
-        super(ColorSpec, self).__init__(Color, default=default, help=help)
+    def __init__(self, default, help=None, key_type=_ExprFieldValueTransform):
+        super(ColorSpec, self).__init__(key_type, Color, default=default, help=help)
 
     @classmethod
     def isconst(cls, val):
@@ -1772,8 +1823,8 @@ class ColorSpec(DataSpec):
         if isinstance(val, string_types):
             return dict(field=val)
 
-        # Must be dict, return as-is
-        return val
+        # Must be dict, return new dict
+        return dict(val)
 
     def prepare_value(self, cls, name, value):
         # Some explanation is in order. We want to accept tuples like
@@ -1793,7 +1844,30 @@ class ColorSpec(DataSpec):
 # DataSpec helpers
 #------------------------------------------------------------------------------
 
-def field(name):
+def expr(expression, transform=None):
+    ''' Convenience function to explicitly return an "expr" specification for
+    a Bokeh :class:`~bokeh.core.properties.DataSpec` property.
+
+    Args:
+        expression (Expression) : a computed expression for a
+            ``DataSpec`` property.
+
+        transform (Transform, optional) : a transform to apply (default: None)
+
+    Returns:
+        dict : ``{ "expr": expression }``
+
+    .. note::
+        This function is included for completeness. String values for
+        property specifications are by default interpreted as field names.
+
+    '''
+    if transform:
+        return dict(expr=expression, transform=transform)
+    return dict(expr=expression)
+
+
+def field(name, transform=None):
     ''' Convenience function to explicitly return a "field" specification for
     a Bokeh :class:`~bokeh.core.properties.DataSpec` property.
 
@@ -1801,6 +1875,8 @@ def field(name):
         name (str) : name of a data source field to reference for a
             ``DataSpec`` property.
 
+        transform (Transform, optional) : a transform to apply (default: None)
+
     Returns:
         dict : ``{ "field": name }``
 
@@ -1809,15 +1885,19 @@ def field(name):
         property specifications are by default interpreted as field names.
 
     '''
+    if transform:
+        return dict(field=name, transform=transform)
     return dict(field=name)
 
-def value(val):
+def value(val, transform=None):
     ''' Convenience function to explicitly return a "value" specification for
     a Bokeh :class:`~bokeh.core.properties.DataSpec` property.
 
     Args:
         val (any) : a fixed value to specify for a ``DataSpec`` property.
 
+        transform (Transform, optional) : a transform to apply (default: None)
+
     Returns:
         dict : ``{ "value": name }``
 
@@ -1836,6 +1916,8 @@ def value(val):
                    text_font_size=value("12pt"), source=source)
 
     '''
+    if transform:
+        return dict(value=val, transform=transform)
     return dict(value=val)
 
 #------------------------------------------------------------------------------
@@ -1901,6 +1983,6 @@ _data_specs = "\n".join(sorted(".. autoclass:: %s" % x.__name__ for x in _find_a
 _containers = "\n".join(sorted(".. autoclass:: %s" % x.__name__ for x in _find_and_remove(ContainerProperty)))
 _basic = "\n".join(sorted(".. autoclass:: %s" % x.__name__ for x in _all_props))
 
-__doc__ = __doc__ % (_basic, _containers, _data_specs)
+__doc__ = format_docstring(__doc__, basic_properties=_basic, container_properties=_containers, dataspec_properties=_data_specs)
 
 del _all_props, _data_specs, _containers, _basic, _find_and_remove
