@@ -4,7 +4,8 @@ import unittest
 from six.moves import xrange
 import copy
 from bokeh.core.properties import List, String, Instance, Dict, Any, Int
-from bokeh.model import Model, _ModelInDocument
+from bokeh.model import Model
+from bokeh.embed import _ModelInDocument
 from bokeh.document import Document
 from bokeh.core.property.containers import PropertyValueList, PropertyValueDict
 from bokeh.util.future import with_metaclass
@@ -14,13 +15,13 @@ def large_plot(n):
     from bokeh.models import (
         Plot, LinearAxis, Grid, GlyphRenderer,
         ColumnDataSource, DataRange1d, PanTool, ZoomInTool, ZoomOutTool, WheelZoomTool, BoxZoomTool,
-        BoxSelectTool, ResizeTool, SaveTool, ResetTool
+        BoxSelectTool, SaveTool, ResetTool
     )
-    from bokeh.models.layouts import VBox
+    from bokeh.models.layouts import Column
     from bokeh.models.glyphs import Line
 
-    vbox = VBox()
-    objects = set([vbox])
+    col = Column()
+    objects = set([col])
 
     for i in xrange(n):
         source = ColumnDataSource(data=dict(x=[0, i + 1], y=[0, i + 1]))
@@ -41,32 +42,31 @@ def large_plot(n):
         wheel_zoom = WheelZoomTool()
         box_zoom = BoxZoomTool()
         box_select = BoxSelectTool()
-        resize = ResizeTool()
         save = SaveTool()
         reset = ResetTool()
-        tools = [pan, zoom_in, zoom_out, wheel_zoom, box_zoom, box_select, resize, save, reset]
+        tools = [pan, zoom_in, zoom_out, wheel_zoom, box_zoom, box_select, save, reset]
         plot.add_tools(*tools)
-        vbox.children.append(plot)
+        col.children.append(plot)
         objects |= set([
-            source, xdr, ydr, plot, xaxis, yaxis, xgrid, ygrid, renderer, glyph,
-            plot.toolbar, plot.tool_events, plot.title, box_zoom.overlay, box_select.overlay] +
+            source, xdr, ydr, plot, xaxis, yaxis, xgrid, ygrid, renderer, renderer.view, glyph, plot.x_scale, plot.y_scale,
+            plot.toolbar, plot.title, box_zoom.overlay, box_select.overlay] +
             tickers + tools)
 
-    return vbox, objects
+    return col, objects
 
 
-class TestViewable(unittest.TestCase):
+class TestMetaModel(unittest.TestCase):
 
     def setUp(self):
-        from bokeh.model import Viewable
-        self.viewable = Viewable
-        self.old_map = copy.copy(self.viewable.model_class_reverse_map)
+        from bokeh.model import MetaModel
+        self.metamodel = MetaModel
+        self.old_map = copy.copy(self.metamodel.model_class_reverse_map)
 
     def tearDown(self):
-        self.viewable.model_class_reverse_map = self.old_map
+        self.metamodel.model_class_reverse_map = self.old_map
 
     def mkclass(self):
-        class Test_Class(with_metaclass(self.viewable)):
+        class Test_Class(with_metaclass(self.metamodel)):
             foo = 1
         return Test_Class
 
@@ -76,10 +76,11 @@ class TestViewable(unittest.TestCase):
         self.assertRaises(Warning, self.mkclass)
 
     def test_get_class(self):
+        from bokeh.model import get_class
         self.mkclass()
-        tclass = self.viewable.get_class('Test_Class')
+        tclass = get_class('Test_Class')
         self.assertTrue(hasattr(tclass, 'foo'))
-        self.assertRaises(KeyError, self.viewable.get_class, 'Imaginary_Class')
+        self.assertRaises(KeyError, get_class, 'Imaginary_Class')
 
 class DeepModel(Model):
     child = Instance(Model)
@@ -124,8 +125,12 @@ class TestModel(unittest.TestCase):
         testObject2 = self.pObjectClass()
         self.assertIsNot(testObject2._id, None)
 
-        self.assertEqual(set(["name", "tags", "js_callbacks"]), testObject.properties())
-        self.assertDictEqual(dict(name=None, tags=[], js_callbacks={}), testObject.properties_with_values(include_defaults=True))
+        self.assertEqual(set(["name", "tags", "js_property_callbacks",
+                              "subscribed_events", "js_event_callbacks"]),
+                         testObject.properties())
+        self.assertDictEqual(dict(name=None, tags=[], js_property_callbacks={},
+                                  js_event_callbacks={}, subscribed_events=[]),
+                             testObject.properties_with_values(include_defaults=True))
         self.assertDictEqual(dict(), testObject.properties_with_values(include_defaults=False))
 
     def test_ref(self):
@@ -199,13 +204,16 @@ class TestModel(unittest.TestCase):
                            "id" : obj._id,
                            "name" : None,
                            "tags" : [],
-                           'js_callbacks': {},
+                           'js_property_callbacks': {},
+                           "js_event_callbacks" : {},
+                           "subscribed_events" : [],
                            "foo" : 42,
                            "bar" : "world" },
                          json)
         self.assertEqual(('{"bar":"world",' +
                           '"child":{"id":"%s","type":"SomeModelToJson"},' +
-                          '"foo":42,"id":"%s","js_callbacks":{},"name":null,"tags":[]}') %
+                          '"foo":42,"id":"%s","js_event_callbacks":{},"js_property_callbacks":{},' +
+                          '"name":null,"subscribed_events":[],"tags":[]}') %
                          (child_obj._id, obj._id),
                          json_string)
 
@@ -277,22 +285,24 @@ class TestModel(unittest.TestCase):
             return counter['value']
         class HasFuncDefaultInt(Model):
             value = Int(default=next_value)
-        obj = HasFuncDefaultInt()
-        self.assertEqual(obj.value, obj.value)
+        obj1 = HasFuncDefaultInt()
+        obj2 = HasFuncDefaultInt()
+        self.assertEqual(obj1.value+1, obj2.value)
 
         # 'value' is a default, but it gets included as a
         # non-default because it's unstable.
-        self.assertTrue('value' in obj.properties_with_values(include_defaults=False))
+        self.assertTrue('value' in obj1.properties_with_values(include_defaults=False))
 
     def test_func_default_with_model(self):
         class HasFuncDefaultModel(Model):
             child = Instance(Model, lambda: Model())
-        obj = HasFuncDefaultModel()
-        self.assertEqual(obj.child._id, obj.child._id)
+        obj1 = HasFuncDefaultModel()
+        obj2 = HasFuncDefaultModel()
+        self.assertNotEqual(obj1.child._id, obj2.child._id)
 
         # 'child' is a default, but it gets included as a
         # non-default because it's unstable.
-        self.assertTrue('child' in obj.properties_with_values(include_defaults=False))
+        self.assertTrue('child' in obj1.properties_with_values(include_defaults=False))
 
 class SomeModelInTestObjects(Model):
     child = Instance(Model)
@@ -301,7 +311,7 @@ class TestModelInDocument(unittest.TestCase):
     def test_single_model(self):
         p = Model()
         self.assertIs(p.document, None)
-        with _ModelInDocument(p):
+        with _ModelInDocument([p]):
             self.assertIsNot(p.document, None)
         self.assertIs(p.document, None)
 
@@ -348,7 +358,7 @@ class TestModelInDocument(unittest.TestCase):
         self.assertIs(p1.document, None)
         self.assertIs(p2.document, None)
 
-    def test_uses_precedent_from_child(self):
+    def test_with_doc_in_child_raises_error(self):
         doc = Document()
         p1 = Model()
         p2 = SomeModelInTestObjects(child=Model())
@@ -356,15 +366,12 @@ class TestModelInDocument(unittest.TestCase):
         self.assertIs(p1.document, None)
         self.assertIs(p2.document, None)
         self.assertIs(p2.child.document, doc)
-        with _ModelInDocument([p1, p2]):
-            self.assertIsNot(p1.document, None)
-            self.assertIsNot(p2.document, None)
-            self.assertIs(p1.document, doc)
-            self.assertIs(p2.document, doc)
-        self.assertIs(p1.document, None)
-        self.assertIs(p2.document, None)
-        self.assertIsNot(p2.child.document, None)
-        self.assertIs(p2.child.document, doc)
+        with self.assertRaisesRegexp(RuntimeError, p2._id):
+            with _ModelInDocument([p1, p2]):
+                self.assertIsNot(p1.document, None)
+                self.assertIsNot(p2.document, None)
+                self.assertIs(p1.document, doc)
+                self.assertIs(p2.document, doc)
 
 class TestContainerMutation(unittest.TestCase):
 
