@@ -3,7 +3,9 @@
 '''
 from importlib import import_module
 import logging
+from subprocess import Popen, PIPE
 
+from ..settings import settings
 
 logger = logging.getLogger(__name__)
 
@@ -48,3 +50,19 @@ def import_required(mod_name, error_msg):
         return import_module(mod_name)
     except ImportError:
         raise RuntimeError(error_msg)
+
+def detect_phantomjs():
+    '''Detect if PhantomJS is avaiable in PATH.'''
+    if settings.phantomjs_path() is not None:
+        phantomjs_path = settings.phantomjs_path()
+    else:
+        phantomjs_path = "phantomjs"
+
+    try:
+        proc = Popen([phantomjs_path, "--version"], stdout=PIPE, stderr=PIPE)
+        proc.wait()
+    except OSError:
+        raise RuntimeError('PhantomJS is not present in PATH. Try "conda install phantomjs" or \
+                           "npm install -g phantomjs-prebuilt"')
+    else:
+        return phantomjs_path
